@@ -223,7 +223,7 @@ sub authorize {
     if ( ref($user2pass) eq 'ARRAY' && ! ref( $user2pass->[0] )) {
 	$default_upw = $user2pass;
     } elsif ( ref($user2pass) eq 'HASH' ) {
-	$realm2upw = %$user2pass;
+	$realm2upw = $user2pass;
     } else {
 	$cb_upw = $user2pass;
     }
@@ -241,16 +241,16 @@ sub authorize {
 	    foreach my $a (@auth) {
 		my $h = $a->{parameter};
 
-		# check if we already have an authorize header for this realm/opaque
+		# check if we already have an authorize header for this realm
 		if ( ! $existing_auth ) {
 		    $existing_auth = {};
 		    foreach my $hdr ( $self->get_header_hashval( $resp )) {
-			my @auth = grep { defined } map { $hdr->{parameter}{$_} }qw( realm opaque );
+			my @auth = grep { defined } map { $hdr->{parameter}{$_} }qw( realm );
 			$existing_auth->{ join( "\0",@auth ) } = 1;
 		    }
 		}
 
-		my @auth = grep { defined } map { $h->{$_} }qw( realm opaque );
+		my @auth = grep { defined } map { $h->{$_} }qw( realm );
 		if ( $existing_auth->{ join( "\0",@auth ) } ) {
 		    # we have this auth header already, don't repeat
 		    next;
@@ -317,6 +317,8 @@ sub authorize {
 		$header.= qq[,cnonce="$digest{cnonce}"] if defined $digest{cnonce};
 		$header.= qq[,qop=$digest{qop}] if defined $digest{qop};
 		$header.= qq[,nc=$digest{nc}] if defined $digest{nc};
+		# Echo back the algorithm if specifically set in response
+		$header.= qq[,algorithm=$h->{algorithm}] if defined $h->{algorithm};
 		$self->add_header( $resp, $header );
 		$auth++;
 	    }
